function [v,alpha,beta] = HOUSE(v)
% [v,alpha,beta] = HOUSE(v)
%
% Zu einem gegebenen Inputvektor v=a berechnet HOUSE den Householdervektor v
% fuer die Householder Transformation von.
% Dabei wird a(2:end) durch v(2:end) ueberschrieben.
%
% Dies ist mathematisch eine Spiegelung auf ein Vielfaches des ersten 
% Einheitsvektors, d.h. v wird so bestimmt, dass
% (I - (2/v'*v) * v*v') * a = -sgn(a_1)*||a||_2 * e1.
%
% Abgespeichert werden dabei v(2:end), alpha, beta fuer die Darstellung
% (I + beta * [1;v(2:end)] * [1;v(2:end)]')  * a = alpha * e1.
%
%
% Es wird nur v(2:end) gendert, v(1)=1 muss selbstndig gendert werden.
% Insbesondere ist es damit erlaubt und beabsichtigt, a=v als Input- und 
% Outputparameter zu benutzen.
% Damit kann bei der QR-Zerlegung der Matrix das Ergebnis sofort wieder in
% die richitge Position geschrieben werden.
%
% -----------------------------------------------------------------------------
%
% Input
% -----
% v      Vektor der Lnge n, der auf ein Vielfaches des Einheitsvektors
%        abgebildet werden soll
%
% -----------------------------------------------------------------------------
%
% Output
% ------
% v      v(2:n) wird ueberschrieben mit dem Householder-Vektor der Spiegelung
%        H=I+beta*[1;v(2:n)]*[1;v(2:n)]'
% alpha  Wert des Inputvektors v=a nach Anwenden der Householder-Transformation
% beta   Parameter beta der Spiegelung H=I+beta*[1;v(2:n)]*[1;v(2:n)]'


