% Script-File

% Hinzufuegen eines Suchpfades fuer M-Files
addpath '../routines/';
addpath '../examples/';


Anz=2;


kmax=6;
message=cell(kmax,1);
message{1}='qr aus MATLAB';
message{2}='qr aus MATLAB mit Spaltenvertauschen';
message{3}='eigener QR';
message{4}='Cholesky fr die Normalengleichung';
message{5}='Vergleich aller';

message{kmax}='ENDE';

k=menu('Auswahl der Verfahren', message);
fprintf('%s\n',message{k});

while k<kmax

      l=menu('Testbeispiel',...
	     'Random-Matrizen',...
	     'Hilbertmatrix',...
	     'Bevlkerungswachstum USA',...
	     'ENDE');
       
      if l==4
	 return
      end
      if l==1
	 % Vektor mit Dimension der Matrix
	 v=[10 50 100 200 300];
      elseif l==2
	 % Vektor mit Dimension der Matrix
	 v=2:7;
      end

      if l==3
	 if k==1 | k==kmax-1
	    figure(1);
	    clf
	    censusgui(1);
	 end
	 if k==2 | k==kmax-1
	    figure(2);
	    clf
	    censusgui(2);
	 end
	 if k==3 | k==kmax-1
	    figure(3);
	    clf
	    censusgui(3);
	 end
	 if k==4 | k==kmax-1
	    figure(4);
	    clf
	    censusgui(4);
	 end
      else % l<=2
	 % Vektoren fuer Zeit, Fehler
	 zeit  =zeros(4,length(v));
	 fehler=zeros(4,length(v));
	 i=1;
	 for n=v
	     fprintf('n=%4d\n',n);
	     for j=1:Anz
		 % Beliebige Matrix der Dimension n
		 if l==1
		    A=randn(2*n,n)-.5;
		    % Beliebige rechte Seite
		    b=randn(2*n,1)-.5;
		 elseif l==2
		    A=hilb(n);
		    % Beliebige rechte Seite
		    x=ones(n,1); b=A*x;
		 end
		 [m,n]=size(A);
	
		 % 1. Variante, qr aus MATLAB
		 if k==1 | k==kmax-1
		    t0=clock;
		    [Q,R]=qr(A);
		    c=Q'*b;
		    c(1:n)=R(1:n,1:n)\c(1:n);
		    zeit(1,i)  =zeit(1,i)  +etime(clock,t0);
		    if l==1
		       fehler(1,i)=fehler(1,i)+norm(c(n+1:m))/norm(c(1:n));
		    else
		       fehler(1,i)=fehler(1,i)+norm(x-c)/norm(x);
		    end
		 end
		 % 2. Variante, qr aus MATLAB mit Spaltenvertauschen
		 if k==2 | k==kmax-1
		    t0=clock;
		    [Q,R,PI]=qr(A);
		    c=Q'*b;
		    c(1:n)=PI*(R(1:n,1:n)\c(1:n));
		    zeit(2,i)  =zeit(2,i)  +etime(clock,t0);
		    if l==1
		       fehler(2,i)=fehler(2,i)+norm(c(n+1:m))/norm(c(1:n));
		    else
		       fehler(2,i)=fehler(2,i)+norm(x-c)/norm(x);
		    end
		 end
		 % 3. Variante, eigene QR-Zerlegung
		 if k==3 | k==kmax-1
		    B=A; y=b;
		    t0=clock;
		    [B,beta]=QRDEC(B);
		    y=QMULT(B,y,beta);
		    y(1:n)=BACKWARD(B(1:n,:),1:n,y(1:n));
		    zeit(3,i)  =zeit(3,i)  +etime(clock,t0);
		    if l==1
		       fehler(3,i)=fehler(3,i)+norm(y(n+1:m))/norm(y(1:n));
		    else
		       fehler(3,i)=fehler(3,i)+norm(x-y)/norm(x);
		    end
		 end % if
		 % 4. Variante, Cholesky fr die Normalengleichung
		 if k==4 | k==kmax-1
		    t0=clock;
		    R=chol(A'*A);
		    y=R\(R'\(A'*b));
		    zeit(4,i)  =zeit(4,i)  +etime(clock,t0);
		    if l==1
		       fehler(4,i)=fehler(4,i)+norm(b-A*y)/norm(y);       
		    else
		       fehler(4,i)=fehler(4,i)+norm(x-y)/norm(x);
		    end
		 end % if
	     end % for j
	       
	     zeit(:,i)  =zeit(:,i)/Anz;
	     fehler(:,i)=fehler(:,i)/Anz;
	     i=i+1;
	 end % for n

	 figure(1);
	 hold off;
	 m=0;
	 clear legendtext
	 if k==1 | k==kmax-1
	    semilogy(v,fehler(1,:),'+r');
	    hold on;
	    m=m+1;
	    legendtext{m}='QR (MATLAB)';
	 end % if
	 if k==2 | k==kmax-1
	    semilogy(v,fehler(2,:),'og');
	    m=m+1;
	    legendtext{m}='QR mit Spaltenvert. (MATLAB)';
	 end % if
	 if k==3 | k==kmax-1
	    semilogy(v,fehler(3,:),'*b');
	    m=m+1;
	    legendtext{m}='eigener QR';
	 end % if
	 if k==4 | k==kmax-1
	    semilogy(v,fehler(4,:),'xm');
	    m=m+1;
	    legendtext{m}='Cholesky Normalengl.';
	 end % if
	 hold off;
	 if l==1
	    title('Norm des Residuums');
	    ylabel('||r||/||x||');
	 else
	    title('Relative Fehler der Lsung');
	    ylabel('||e||/||x||');
	 end
	 xlabel('n');
	 legend(legendtext);
	 
	 figure(2);
	 hold off;
	 if k==1 | k==kmax-1
	    plot(v,zeit(1,:),'+r');
	    hold on;
	 end % if
	 if k==2 | k==kmax-1
	    plot(v,zeit(2,:),'og');
	 end % if
	 if k==3 | k==kmax-1
	    plot(v,zeit(3,:),'*b');
	 end % if
	 if k==4 | k==kmax-1
	    plot(v,zeit(4,:),'xm');
	 end % if
	 hold off;
	 title('Rechenzeit');
	 xlabel('n');
	 ylabel('[sec]');
	 legend(legendtext);
	 
      end % if-else

      k=menu('Auswahl der Verfahren', message);
      fprintf('%s\n',message{k});
end % while



